/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * This source file adapted from:
 * http://pcbunn.cacr.caltech.edu/Java/Genetic.html
 * 
 */

package net.jasonfreeman.itsm.analysis.genetic;

import net.jasonfreeman.itsm.analysis.Chunk;
import net.jasonfreeman.itsm.analysis.fft.FFTHelper;
import net.jasonfreeman.itsm.util.ErrorHandler;
import net.jasonfreeman.itsm.util.SettingsManager;

public class CachingCostMatrix {

	private double[][] costs;
	
	public CachingCostMatrix(Chunk[] chunks) {
		int length = chunks.length;
		costs = new double[length][length];
		for (int i=0; i < length; i++) {
			for (int j=0; j < length; j++) {
				if (i == j)
					costs[i][j] = 0;
				else if (i > j)
					costs[i][j] = costs[j][i];
				else
					try {
						if (SettingsManager.getInstance().isFastMode()) {
							costs[i][j] = FFTHelper.doFFTdiff(chunks[i].getFftAvg(), chunks[j].getFftAvg());
						} else 
							costs[i][j] = FFTHelper.doFFTdiffWithD(chunks[i].getFftAvg(), chunks[j].getFftAvg(), chunks[i].getFftDAvg(), chunks[j].getFftDAvg());
					} catch (Exception e) {
						costs[i][j] = Double.MAX_VALUE;
						ErrorHandler.getInstance().handleError(e);
					}
			}
		}
	}
	
	public double getCost(int i, int j) {
		return costs[i][j];
	}
	
}
