package net.jasonfreeman.itsm.analysis.genetic;
/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * This source file adapted from:
 * http://pcbunn.cacr.caltech.edu/Java/Genetic.html
 * 
 */

import net.jasonfreeman.itsm.analysis.Chunk;
import net.jasonfreeman.itsm.gui.ProgressManager;
import net.jasonfreeman.itsm.util.SettingsManager;


public class Genetic {
	int Initial_population = 800;
	int Mating_population = Initial_population/2;
	int Favoured_population = Mating_population/2;
	int cut_length;
	double Mutation_probability = 0.10;

	int Epoch;
	double min_cost;
	double costPerfect;

	Chromosome [] chromosomes;
	CachingCostMatrix costMatrix;
	
	SettingsManager settings = SettingsManager.getInstance();
	ProgressManager progress = ProgressManager.getInstance();

   public Chunk[] optimizeChunks(Chunk[] chunks) {
   	   cut_length = chunks.length / 5;
   	   min_cost = chunks.length;
   	   costPerfect = chunks.length;
   	   costMatrix = new CachingCostMatrix(chunks);
	   Chromosome perfect = new Chromosome(chunks, costMatrix);
	   int cl [] = new int[chunks.length];
	   for(int i=0;i<chunks.length;i++)
	   		cl[i] = i;
	   perfect.set_cities(cl);
	   perfect.calculate_cost(chunks, costMatrix);
	   costPerfect = perfect.get_cost();
	   
	   // generate an initial population of chromosomes
	   chromosomes = new Chromosome[Initial_population];
	   for(int i=0;i<Initial_population;i++) {
		   chromosomes[i] = new Chromosome(chunks, costMatrix);
		   chromosomes[i].set_cut(cut_length);
		   chromosomes[i].set_mutation(Mutation_probability);
	   }

	   Sort_chromosomes(Initial_population);

	  Epoch = 0;
	  solve(chunks, costMatrix);
	  
	  int[] bestFit = chromosomes[0].chunk_list;
	  Chunk[] newOrder = new Chunk[bestFit.length];
	  for (int i=0; i < bestFit.length; i++) {
	  	newOrder[i] = chunks[bestFit[i]];
	  }
	  return newOrder;
   }

   private void Sort_chromosomes(int num) {
	   Chromosome ctemp;
	   boolean swapped = true;
	   while(swapped) {
		   swapped = false;
		   for(int i=0;i<num-1;i++) {
			   if(chromosomes[i].get_cost() > chromosomes[i+1].get_cost() ) {
				   ctemp = chromosomes[i];
				   chromosomes[i] = chromosomes[i+1];
				   chromosomes[i+1] = ctemp;
				   swapped = true;
			   }
		   }
	   }
   }

   private void solve(Chunk[] chunks, CachingCostMatrix costMatrix) {

	   double this_cost = 500.0;
	   double old_cost = 0.0;
	   int count_same = 0;

	   while(this_cost > costPerfect && count_same < 100) {

		   Epoch++;
		   if ((Epoch % 100) == 0) {
			   progress.setPercentInfo(Epoch / 10000);
		   }
		   
		   int ioffset = Mating_population;
		   int mutated = 0;


		   // Mate the chromosomes in the favoured population with all in the mating population
		   for(int i=0;i<Favoured_population;i++) {
			   Chromosome cmother = chromosomes[i];
			   // Select partner from the mating population
			   int father = (int) ( 0.999999*Math.random()*(double)Mating_population);
			   Chromosome cfather = chromosomes[father];

			   mutated += cmother.mate(cfather,chromosomes[ioffset],chromosomes[ioffset+1]);
			   ioffset += 2;
		   }

		   // The new generation of chromosomes is in position Mating_population ... move them
		   // to the right place in the list and calculate their costs

		   for(int i=0;i<Mating_population;i++) {
			   chromosomes[i] = chromosomes[i+Mating_population];
			   chromosomes[i].calculate_cost(chunks, costMatrix);
		   }

		   // Now sort them

		   Sort_chromosomes(Mating_population);

		   double cost = chromosomes[0].get_cost();
		   this_cost = cost;

		   if((int)this_cost == (int)old_cost) {
			   count_same++;
		   } else {
			   count_same = 0;
			   old_cost = this_cost;
		   }
	   }
   }
}

