/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.gui;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import net.jasonfreeman.itsm.ITSM;
import net.jasonfreeman.itsm.util.ErrorHandler;

/**
 * Display error message and stack trace to user.
 * 
 * @author jason
 *
 */

public class ErrorPanel extends JPanel implements ActionListener {

	JButton okButton;

	ITSM applet;
	Container contentPane;

	public ErrorPanel(ITSM applet, String s) {
		super();
		this.applet = applet;
		contentPane = applet.getContentPane();
		setBackground(Color.white);

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		add(Box.createVerticalGlue());
		add(Box.createRigidArea(new Dimension(0, 25)));

		JPanel textPanel = new JPanel();
		textPanel.setBackground(Color.white);
		textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.X_AXIS));
		textPanel.add(Box.createRigidArea(new Dimension(50, 0)));
		JLabel label = new JLabel("An error has occurred:");
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		textPanel.add(label);
		textPanel.add(Box.createHorizontalGlue());
		add(textPanel);
		add(Box.createVerticalGlue());
		add(new HorizontalSeparator());
		add(Box.createVerticalGlue());

		JPanel infoPanel = new JPanel();
		infoPanel.setBackground(Color.white);
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.X_AXIS));
		infoPanel.add(Box.createRigidArea(new Dimension(50, 0)));
		JEditorPane info = null;
		info = new JEditorPane("text/plain", s);
		info.setEditable(false);
		info.setBackground(Color.white);
		info.setOpaque(false);
		info.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		JScrollPane editorScrollPane = new JScrollPane(info);
		editorScrollPane
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane.setBackground(Color.white);
		editorScrollPane.setPreferredSize(new Dimension(400, 300));
		infoPanel.add(editorScrollPane);
		infoPanel.add(Box.createRigidArea(new Dimension(50, 0)));
		info.setCaretPosition(0);
		add(infoPanel);
		
		add(Box.createVerticalGlue());
		add(new HorizontalSeparator());
		add(Box.createVerticalGlue());

		JPanel okPanel = new JPanel();
		okPanel.setBackground(Color.white);
		okPanel.setLayout(new BoxLayout(okPanel, BoxLayout.X_AXIS));
		okPanel.add(Box.createHorizontalGlue());
		okButton = new JButton("OK");
		okButton.addActionListener(this);
		okButton.setBackground(Color.white);
		okPanel.add(okButton);
		okPanel.add(Box.createHorizontalGlue());
		add(okPanel);
		add(Box.createVerticalGlue());

		applet.setContentPane(this);
		applet.setContentPaneLocked(true); // lock content pane so error message stays until user hits ok
		if (contentPane != null)
			applet.setContentPane(contentPane);
		applet.validate();
	}

	public void actionPerformed(ActionEvent e) {
		try {
			if (e.getSource() == okButton) {
				applet.setContentPaneLocked(false); // switch to whatever panel is waiting to be shown
			}
		} catch (RuntimeException e1) {
			ErrorHandler.getInstance().handleError(e1);
		}
	}
}
