/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.jasonfreeman.itsm.ITSM;
import net.jasonfreeman.itsm.util.ErrorHandler;
import net.jasonfreeman.itsm.util.SettingsManager;

/**
 * Configure all the parameters of the signature-making algorithm.
 * 
 * @author jason
 *
 */

public class SongSelectionPanel extends JPanel implements ActionListener, ItemListener {

    private SettingsManager settings = SettingsManager.getInstance();
    private JComboBox critList, maxSongList, winList, mixList, fastList, limitModeList;
    private JLabel infoLabel;
    private JCheckBox ignorePodcast, ignoreVideo;
    private JButton continueButton;
    private ITSM applet;
    
    public SongSelectionPanel(ITSM applet) {
        this.applet = applet;
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        add(Box.createVerticalGlue());
        add(Box.createVerticalGlue());
        setBackground(Color.white);
        
        JPanel labelPanel = new JPanel();
        labelPanel.setBackground(Color.white);
        labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.X_AXIS));
        labelPanel.add(Box.createRigidArea(new Dimension(50, 0)));
        JLabel label = new JLabel("Please set the parameters of the signature-making algorithm.");
        label.setFont(label.getFont().deriveFont(Font.BOLD));
        labelPanel.add(label);
        labelPanel.add(Box.createHorizontalGlue());
        add(labelPanel);
        
        add(Box.createVerticalGlue());
        add(new HorizontalSeparator());
        add(Box.createVerticalGlue());
        
        JPanel maxPanel = new JPanel();
        maxPanel.setBackground(Color.white);
        maxPanel.setLayout(new BoxLayout(maxPanel, BoxLayout.X_AXIS));
        maxPanel.add(Box.createRigidArea(new Dimension(50,0)));
    		label = new JLabel("Select up to");
        maxPanel.add(label);
        maxPanel.add(Box.createRigidArea(new Dimension(15,0)));
        String[] maxSongVals = {"10", "20", "30", "40", "50", "60", "70", "80", "90", "100"};
        maxSongList = new JComboBox(maxSongVals);
        maxSongList.setBackground(Color.white);
        int selectedIndex = settings.getMaxSongs() / 10 - 1;
        if (selectedIndex < maxSongVals.length)
        		maxSongList.setSelectedIndex(selectedIndex);
        maxSongList.addActionListener(this);
        maxPanel.add(maxSongList);
        maxPanel.add(Box.createRigidArea(new Dimension(15,0)));
        label = new JLabel("total songs.");
        maxPanel.add(label);
        maxPanel.add(Box.createRigidArea(new Dimension(200,0)));
        maxPanel.add(Box.createHorizontalGlue());
        add(maxPanel);

        add(Box.createVerticalGlue());
        
        JPanel limitPanel = new JPanel();
        limitPanel.setBackground(Color.white);
        limitPanel.setLayout(new BoxLayout(limitPanel, BoxLayout.X_AXIS));
        limitPanel.add(Box.createRigidArea(new Dimension(50,0)));
    		label = new JLabel("Select up to one song from each");
    		limitPanel.add(label);
    		limitPanel.add(Box.createRigidArea(new Dimension(15,0)));
        String[] limitVals = {"artist.", "album.", "album or artist.", "(no limits)."};
        limitModeList = new JComboBox(limitVals);
        limitModeList.setBackground(Color.white);
        selectedIndex = settings.getLimitMode();
        limitModeList.setSelectedIndex(selectedIndex);
        limitModeList.addActionListener(this);
        limitPanel.add(limitModeList);
        limitPanel.add(Box.createRigidArea(new Dimension(15,0)));
        limitPanel.add(Box.createRigidArea(new Dimension(200,0)));
        limitPanel.add(Box.createHorizontalGlue());
        add(limitPanel);

        add(Box.createVerticalGlue());
        
        JPanel ignorePanel = new JPanel();
        ignorePanel.setBackground(Color.white);
        ignorePanel.setLayout(new BoxLayout(ignorePanel, BoxLayout.X_AXIS));
        ignorePanel.add(Box.createRigidArea(new Dimension(50, 0)));
        ignorePodcast = new JCheckBox("ignore podcasts", SettingsManager.getInstance().isIgnorePodcasts());
        ignorePodcast.addItemListener(this);
        ignorePodcast.setBackground(Color.white);
        ignorePanel.add(ignorePodcast);
        ignorePanel.add(Box.createHorizontalGlue());
        ignoreVideo = new JCheckBox("ignore videos", SettingsManager.getInstance().isIgnoreVideos());
        ignoreVideo.addItemListener(this);
        ignoreVideo.setBackground(Color.white);
        ignorePanel.add(ignoreVideo);
        ignorePanel.add(Box.createHorizontalGlue());
        ignorePanel.add(Box.createHorizontalGlue());
        add(ignorePanel);
        
        add(Box.createVerticalGlue());
        
        JPanel critPanel = new JPanel();
        critPanel.setBackground(Color.white);
        critPanel.setLayout(new BoxLayout(critPanel, BoxLayout.X_AXIS));
        critPanel.add(Box.createRigidArea(new Dimension(50,0)));
        label = new JLabel("Choose songs based on");
        critPanel.add(label);
        critPanel.add(Box.createRigidArea(new Dimension(15,0)));
        String[] criteria = new String[4];
        criteria[SettingsManager.TIMES_PLAYED] = "play count.";
        criteria[SettingsManager.LAST_DATE_PLAYED] = "last play date.";
        criteria[SettingsManager.DATE_ADDED] =	"date added to iTunes.";
        criteria[SettingsManager.MY_RATING] = "my rating.";
        critList = new JComboBox(criteria);
        critList.setBackground(Color.white);
        critList.setSelectedIndex(settings.getSelectionMode());
        critList.addActionListener(this);
        critPanel.add(critList);
        critPanel.add(Box.createRigidArea(new Dimension(200,0)));
        critPanel.add(Box.createHorizontalGlue());
        add(critPanel);
        
        add(Box.createVerticalGlue());
        
        JPanel winPanel = new JPanel();
        winPanel.setBackground(Color.white);
        winPanel.setLayout(new BoxLayout(winPanel, BoxLayout.X_AXIS));
        winPanel.add(Box.createRigidArea(new Dimension(50, 0)));
        label = new JLabel("On average, extract a");
        winPanel.add(label);
        winPanel.add(Box.createRigidArea(new Dimension(15,0)));
        String[] winDurs = new String[12];
        for (int i=0; i < winDurs.length; i++) {
        		winDurs[i] = String.valueOf((i + 1) / 2.0);
        }
        winList = new JComboBox(winDurs);
        winList.setBackground(Color.white);
        int index = (int) (settings.getWindowSizeInSeconds(0.5f) * 2 - 1);
        if (index < winDurs.length)
        		winList.setSelectedIndex(index);
        winList.addActionListener(this);
        winPanel.add(winList);
        winPanel.add(Box.createRigidArea(new Dimension(15,0)));
        label = new JLabel("second chunk of each selected song.");
        winPanel.add(label);
        winPanel.add(Box.createRigidArea(new Dimension(200,0)));
        winPanel.add(Box.createHorizontalGlue());
        add(winPanel);
        
        add(Box.createVerticalGlue());
        
        JPanel mixPanel = new JPanel();
        mixPanel.setBackground(Color.white);
        mixPanel.setLayout(new BoxLayout(mixPanel, BoxLayout.X_AXIS));
        mixPanel.add(Box.createRigidArea(new Dimension(50,0)));
        label = new JLabel("Layer up to");
        mixPanel.add(label);
        mixPanel.add(Box.createRigidArea(new Dimension(15,0)));
        String [] mixNums = new String[9];
        for (int i=0; i < mixNums.length; i++) {
        		mixNums[i] = String.valueOf(i + 2);
        }
        mixList = new JComboBox(mixNums);
        mixList.setBackground(Color.white);
        index = (int) (settings.getMaxSimul() - 2);
        if (index < mixNums.length)
        		mixList.setSelectedIndex(index);
        mixList.addActionListener(this);
        mixPanel.add(mixList);
        mixPanel.add(Box.createRigidArea(new Dimension(15,0)));
        label = new JLabel("chunks simultaneously.");
        mixPanel.add(label);
        mixPanel.add(Box.createRigidArea(new Dimension(200,0)));
        mixPanel.add(Box.createHorizontalGlue());
        add(mixPanel);
        
        add(Box.createVerticalGlue());
        
        JPanel fastPanel = new JPanel();
        fastPanel.setBackground(Color.white);
        fastPanel.setLayout(new BoxLayout(fastPanel, BoxLayout.X_AXIS));
        fastPanel.add(Box.createRigidArea(new Dimension(50,0)));
    		label = new JLabel("Optimize algorithm for");
    		fastPanel.add(label);
    		fastPanel.add(Box.createRigidArea(new Dimension(15,0)));
         String[] fastVals = {"fastest operation.", "best results."};
        fastList = new JComboBox(fastVals);
        fastList.setBackground(Color.white);
        if (settings.isFastMode())
        		fastList.setSelectedIndex(0);
        else
        		fastList.setSelectedIndex(1);
        fastList.addActionListener(this);
        fastPanel.add(fastList);
        fastPanel.add(Box.createRigidArea(new Dimension(200,0)));
        fastPanel.add(Box.createHorizontalGlue());
        add(fastPanel);
        
        add(Box.createVerticalGlue());
        
        JPanel infoPanel = new JPanel();
        infoPanel.setBackground(Color.white);
        infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.X_AXIS));
        infoPanel.add(Box.createRigidArea(new Dimension(50,0)));
        infoLabel = new JLabel("");
        setInfoLabel();
        infoPanel.add(infoLabel);
        infoPanel.add(Box.createHorizontalGlue());
        add(infoPanel);
        
        add(Box.createVerticalGlue());
        add(new HorizontalSeparator());
        add(Box.createVerticalGlue());

        JPanel continuePanel = new JPanel();
        continuePanel.setBackground(Color.white);
        continuePanel.setLayout(new BoxLayout(continuePanel, BoxLayout.X_AXIS));
        continuePanel.add(Box.createHorizontalGlue());
        continueButton = new JButton("continue");
        continueButton.setBackground(Color.white);
        continueButton.addActionListener(this);
        continuePanel.add(continueButton);
        continuePanel.add(Box.createHorizontalGlue());
        add(continuePanel);
        
        add(Box.createVerticalGlue());
        
        applet.setContentPane(this);
        applet.validate();
     }

    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        try {
			if (e.getSource() == critList) {
			    settings.setSelectionMode(critList.getSelectedIndex());
			} else if (e.getSource() == maxSongList) {
					settings.setMaxSongs((maxSongList.getSelectedIndex() + 1) * 10);
			} else if (e.getSource() == winList) {
					settings.setWindowSizeInSeconds((winList.getSelectedIndex() + 1) * 0.5);
			} else if (e.getSource() == mixList) {
					settings.setMaxSimul(mixList.getSelectedIndex() + 2);
			} else if (e.getSource() == fastList) {
					if (fastList.getSelectedIndex() == 0)
						settings.setFastMode(true);
					else
						settings.setFastMode(false);
			} else if (e.getSource() == continueButton) {
				new SavePanel(applet);
			} else if (e.getSource() == limitModeList) {
				settings.setLimitMode(limitModeList.getSelectedIndex());
			}
			setInfoLabel();
		} catch (Throwable e1) {
			ErrorHandler.getInstance().handleError(e1);
		}
    }

    /**
     * Update display estimating signature length.
     *
     */
    private void setInfoLabel() {
    		int dur = (int) Math.round(settings.getMaxSongs() * settings.getWindowSizeInSeconds(0.5f) / settings.getMaxSimul());
    		infoLabel.setText("Your iTunes signature will be approximately " + dur + " seconds long.");
    }

	public void itemStateChanged(ItemEvent e) {
		try {
			if (e.getSource() == ignorePodcast) {
				settings.setIgnorePodcasts(e.getStateChange() == ItemEvent.SELECTED);
			} else if (e.getSource() == ignoreVideo) {
				settings.setIgnoreVideos(e.getStateChange() == ItemEvent.SELECTED);
			}
		} catch (Throwable e1) {
			ErrorHandler.getInstance().handleError(e1);
		}
	}
}
