/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.gui;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import net.jasonfreeman.itsm.ITSM;
import net.jasonfreeman.itsm.analysis.Chunk;
import net.jasonfreeman.itsm.selection.Song;
import net.jasonfreeman.itsm.util.ErrorHandler;

/**
 * Show the contents of the signature.
 * 
 * @author jason
 *
 */

public class StructurePanel extends JPanel implements ActionListener {

	JButton okButton;

	ITSM applet;
	
	Container contentPane;

	public StructurePanel(ITSM applet, Chunk[] chunks) {
		super();
		this.applet = applet;
		setBackground(Color.white);

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		add(Box.createVerticalGlue());

		JPanel textPanel = new JPanel();
		textPanel.setBackground(Color.white);
		textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.X_AXIS));
		textPanel.add(Box.createRigidArea(new Dimension(50, 0)));
		JLabel label = new JLabel("Here's what's in your signature (by order of appearance):");
        label.setFont(label.getFont().deriveFont(Font.BOLD));
		textPanel.add(label);
		textPanel.add(Box.createHorizontalGlue());
		add(textPanel);
		add(Box.createVerticalGlue());
		add(new HorizontalSeparator());
		add(Box.createVerticalGlue());

		JPanel infoPanel = new JPanel();
		infoPanel.setBackground(Color.white);
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.X_AXIS));
		infoPanel.add(Box.createRigidArea(new Dimension(50, 0)));
		JEditorPane info = null;
		info = new JEditorPane("text/html", displayStructure(chunks));
		info.setEditable(false);
		info.setBackground(Color.white);
		info.setOpaque(false);
		info.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		JScrollPane editorScrollPane = new JScrollPane(info);
		editorScrollPane
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane.setBackground(Color.white);
		editorScrollPane.setPreferredSize(new Dimension(400, 300));
		infoPanel.add(editorScrollPane);
		infoPanel.add(Box.createRigidArea(new Dimension(50, 0)));
		info.setCaretPosition(0);
		add(infoPanel);
		
		add(Box.createVerticalGlue());
		add(new HorizontalSeparator());
		add(Box.createVerticalGlue());

		JPanel okPanel = new JPanel();
		okPanel.setBackground(Color.white);
		okPanel.setLayout(new BoxLayout(okPanel, BoxLayout.X_AXIS));
		okPanel.add(Box.createHorizontalGlue());
		okButton = new JButton("OK");
		okButton.addActionListener(this);
		okButton.setBackground(Color.white);
		okPanel.add(okButton);
		okPanel.add(Box.createHorizontalGlue());
		add(okPanel);
		add(Box.createVerticalGlue());

		contentPane = applet.getContentPane();
		applet.setContentPane(this);
		applet.validate();
	}

	public void actionPerformed(ActionEvent e) {
		try {
			if (e.getSource() == okButton) {
				applet.setContentPane(contentPane);
			}
		} catch (Throwable e1) {
			ErrorHandler.getInstance().handleError(e1);
		}
	}
	
	/**
	 * Turn the chunk array into an HTML table for display.
	 * 
	 * @param chunks
	 * @return the HTML code
	 */
	private String displayStructure(Chunk[] chunks) {
		StringBuffer sb = new StringBuffer();
		sb.append("<html><table width=\"100%\"><tr><td><b>Title</b></td><td><b>Artist</b></td><td><b>Album</b></td><td><b>Starting At</b></td><td><b>Ending At</b></td></tr>");
		for (int i=0; i < chunks.length; i++) {
			sb.append("<tr><td colspan=\"5\"><hr></td></tr>");
			Chunk chunk = chunks[i];
			Song song = chunk.getSong();
			String name = song.getName();
			String artist = song.getArtist();
			String album = song.getAlbum();
			int dur = chunk.getDur();
			int startTime = chunk.getStartTime();
			sb.append("<tr><td>");
			sb.append(name);
			sb.append("</td><td>");
			sb.append(artist);
			sb.append("</td><td>");
			sb.append(album);
			sb.append("</td><td>");
			sb.append(qtTimeToString(startTime, chunk.getTimeBase()));
			sb.append("</td><td>");
			sb.append(qtTimeToString(startTime + dur, chunk.getTimeBase()));
			sb.append("</td></tr>");
		}
		sb.append("</table></html>");
		return sb.toString();
	}
	
	/**
	 * 
	 * 
	 * @param time quicktime time in timescale
	 * @param timeBase time scale
	 * @return time as string in minutes and seconds
	 */
	private String qtTimeToString(int time, int timeBase) {
		int minutes = time / (60 * timeBase);
		int seconds = (time / timeBase) - (minutes * 60);
		int tenths = (time / (timeBase / 10)) - (minutes * 600) - (seconds * 10);
		if (seconds < 10)
			return (minutes + ":0" + seconds + "." + tenths);
		else
			return (minutes + ":" + seconds + "." + tenths);
	}
}
