/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.filechooser.FileFilter;

import net.jasonfreeman.itsm.ITSM;
import net.jasonfreeman.itsm.util.ErrorHandler;
import net.jasonfreeman.itsm.util.SettingsManager;

/**
 * This panel is used only when we can't find the iTunes Music Library file in
 * the expected location. It asks the user to locate it themselves.
 * 
 * @author jason
 *
 */

public class iTunesXMLPanel extends JPanel implements ActionListener {

    private JLabel filePath;
    private JButton select, continueButton;
    private SettingsManager settings = SettingsManager.getInstance();
    private ITSM applet;
    
    public iTunesXMLPanel(ITSM applet) {
    		this.applet = applet;
        setBackground(Color.white);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(Box.createVerticalGlue());
		add(Box.createVerticalGlue());
        
        JPanel labelPanel = new JPanel();
        labelPanel.setBackground(Color.white);
        labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.X_AXIS));
        labelPanel.add(Box.createRigidArea(new Dimension(50, 0)));
        JLabel label = new JLabel("Please locate your iTunes data file.");
        label.setFont(label.getFont().deriveFont(Font.BOLD));
        labelPanel.add(label);
        labelPanel.add(Box.createHorizontalGlue());
        add(labelPanel);
		
        add(Box.createVerticalGlue());
        add(new HorizontalSeparator());
        add(Box.createVerticalGlue());
        
        JEditorPane info;
        JPanel htmlPanel = new JPanel();
        htmlPanel.setBackground(Color.white);
        htmlPanel.setLayout(new BoxLayout(htmlPanel, BoxLayout.X_AXIS));
        htmlPanel.add(Box.createRigidArea(new Dimension(50, 0)));
        try {
    			info = new JEditorPane(this.getClass().getResource("/html/xmlinfo.html"));
        } catch (Exception e) {
    			info = new JEditorPane("text/plain", "Error: Could not locate info file.");
    		}
    		info.setEditable(false);
    		info.setBackground(Color.white);
    		info.setOpaque(false);
    		info.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    		JScrollPane editorScrollPane = new JScrollPane(info);
    		editorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
    		editorScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    		editorScrollPane.setPreferredSize(new Dimension(400,200));
    		editorScrollPane.setBackground(Color.white);
    		htmlPanel.add(editorScrollPane);
    		htmlPanel.add(Box.createRigidArea(new Dimension(50, 0)));
    		add(htmlPanel);

    		add(Box.createVerticalGlue());
    		JPanel infoPanel = new JPanel();
    		infoPanel.setBackground(Color.white);
    		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.X_AXIS));
    		infoPanel.add(Box.createRigidArea(new Dimension(50, 0)));
         label = new JLabel("Location of iTunes Music Library file:");
         infoPanel.add(label);
         infoPanel.add(Box.createHorizontalGlue());
         add(infoPanel);
         add(Box.createVerticalGlue());
 
    		JPanel selectPanel = new JPanel();
    		selectPanel.setBackground(Color.white);
    		selectPanel.setLayout(new BoxLayout(selectPanel, BoxLayout.X_AXIS));
    		selectPanel.add(Box.createRigidArea(new Dimension(50,0)));
        select = new JButton("Browse...");
        select.setBackground(Color.white);
        select.addActionListener(this);
        selectPanel.add(select);
        selectPanel.add(Box.createRigidArea(new Dimension(15,0)));
        filePath = new JLabel("not found!");
        selectPanel.add(filePath);
        selectPanel.add(Box.createHorizontalGlue());
        add(selectPanel);
        add(Box.createVerticalGlue());
        add(new HorizontalSeparator());
        add(Box.createVerticalGlue());
        
        JPanel continuePanel = new JPanel();
        continuePanel.setBackground(Color.white);
        continuePanel.setLayout(new BoxLayout(continuePanel, BoxLayout.X_AXIS));
        continuePanel.add(Box.createHorizontalGlue());
        continueButton = new JButton("continue");
        continueButton.setBackground(Color.white);
        continueButton.addActionListener(this);
        continueButton.setEnabled(false); // don't let them continue until they've found the file
        continuePanel.add(continueButton);
        continuePanel.add(Box.createHorizontalGlue());
        add(continuePanel);
        add(Box.createVerticalGlue());
        
        applet.setContentPane(this);
        applet.validate();
    }

    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        try {
			if (e.getSource() == select) {
			    JFileChooser fc = new JFileChooser();
			    fc.setFileFilter(new FileFilter() {

			        public boolean accept(File f) {
			            return (f.getName().equals("iTunes Music Library.xml") ||
			                    f.isDirectory());
			        }

			        public String getDescription() {
			            return ("iTunes Music Library files");
			        }
			    });
			    int returnVal = fc.showDialog(this, "Select");
			    if (returnVal == JFileChooser.APPROVE_OPTION) {
			        File file = fc.getSelectedFile();
			        if (!file.getName().equals("iTunes Music Library.xml")) {
			        		JOptionPane.showMessageDialog(this,
			        				"Invalid file! Choose a file named \"iTunes Music Library.xml\".",
									"File type error",
			        		    JOptionPane.ERROR_MESSAGE);
			        		return;
			        }
			        settings.setiTunesXmlFile(file.getAbsolutePath());
			        continueButton.setEnabled(true);
			        filePath.setText(file.getParent());
			    }
			} else if (e.getSource() == continueButton) {
				new SongSelectionPanel(applet);
			}
		} catch (Throwable e1) {
			ErrorHandler.getInstance().handleError(e1);
		}
    }

}
