/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.mixing;

import net.jasonfreeman.itsm.util.SettingsManager;
import quicktime.QTException;
import quicktime.std.StdQTConstants;
import quicktime.std.movies.Atom;
import quicktime.std.movies.AtomContainer;
import quicktime.std.movies.Movie;
import quicktime.std.movies.Track;
import quicktime.std.movies.TweenData;
import quicktime.std.movies.media.DataRef;
import quicktime.std.movies.media.Media;
import quicktime.std.movies.media.SampleDescription;
import quicktime.std.movies.media.SoundDescription;
import quicktime.std.movies.media.TweenMedia;
import quicktime.util.EndianOrder;
import quicktime.util.QTHandle;

/**
 * Create the tween track to do a volume fadein/fadeout
 * 
 * Beware, this is probably the nastiest code in the whole program.
 * 
 * @author jason
 *
 */

public class TweenTrack {

	// NB: we keep all these Atom references around because
	// otherwise Quicktime has a tendency to garbage collect them while they're
	// still in use...
	private AtomContainer silenceContainer, fullContainer;

	private Atom silenceAtom, fullAtom;

	private Atom temp1, temp2, tempAtom5, tempAtom6, silenceChildAtom, fullChildAtom, inputMapChildAtom;

	private TweenData silenceData, fullData;

	private Atom[] tempAtoms3, tempAtoms4, fadeInAtoms, tempAtoms7, tempAtoms8,
			fadeOutAtoms, fadeInChildAtoms, fadeOutChildAtoms;

	private AtomContainer[] fadeInContainers, fadeOutContainers;

	private TweenData[] fadeInData, fadeOutData;

	private Track faderTrack;

	private Media faderMedia;

	private DataRef ref;

	private QTHandle handle;
	
	private AtomContainer inputMap;

	private Atom inputAtom, inputTemp1, inputTemp2;
	
	private SampleDescription sampleDescription;

	/**
	 * @param result
	 * @param destTrack
	 * @param destStart
	 * @param dur
	 * @throws QTException
	 */
	public TweenTrack(Movie result, Track destTrack, int destStart, int dur)
			throws QTException {
		double fadePercent = SettingsManager.getInstance().getFadePercent();
		int exponentialSteps = dur / 30;
		if (exponentialSteps == 0)
			exponentialSteps = 1;

		// start silence
		silenceContainer = new AtomContainer();
		silenceData = new TweenData(EndianOrder
				.flipNativeToBigEndian16((short) 0), EndianOrder
				.flipNativeToBigEndian16((short) 0));
		silenceChildAtom = new Atom(
				StdQTConstants.kParentAtomIsContainer);
		silenceAtom = silenceContainer.insertChild(silenceChildAtom,
				StdQTConstants.kTweenEntry, 1, 0);
		temp1 = silenceContainer
				.insertChild(
						silenceAtom,
						StdQTConstants.kTweenType,
						1,
						0,
						EndianOrder
								.flipNativeToBigEndian32(StdQTConstants.kTweenTypeShort));
		temp2 = silenceContainer.insertChild(silenceAtom,
				StdQTConstants.kTweenData, 1, 0, silenceData);
		silenceContainer.lock();

		// fade in
		fadeInContainers = new AtomContainer[exponentialSteps];
		fadeInData = new TweenData[fadeInContainers.length];
		fadeInAtoms = new Atom[fadeInContainers.length];
		tempAtoms3 = new Atom[fadeInContainers.length];
		tempAtoms4 = new Atom[fadeInContainers.length];
		fadeInChildAtoms = new Atom[fadeInContainers.length];
		for (int i = 0; i < fadeInContainers.length; i++) {
			fadeInContainers[i] = new AtomContainer();
			short lo = EndianOrder.flipNativeToBigEndian16((short) ((Math.pow(
					3, i / (double) exponentialSteps) - 1) * 2550 * 0.5));
			short hi = EndianOrder.flipNativeToBigEndian16((short) ((Math.pow(
					3, (i + 1) / (double) exponentialSteps) - 1) * 2550 * 0.5));
			fadeInData[i] = new TweenData(lo, hi);
			fadeInChildAtoms[i] = new Atom(
					StdQTConstants.kParentAtomIsContainer);
			fadeInAtoms[i] = fadeInContainers[i].insertChild(fadeInChildAtoms[i],
					StdQTConstants.kTweenEntry, 1, 0);
			tempAtoms3[i] = fadeInContainers[i]
					.insertChild(
							fadeInAtoms[i],
							StdQTConstants.kTweenType,
							1,
							0,
							EndianOrder
									.flipNativeToBigEndian32(StdQTConstants.kTweenTypeShort));
			tempAtoms4[i] = fadeInContainers[i].insertChild(fadeInAtoms[i],
					StdQTConstants.kTweenData, 1, 0, fadeInData[i]);
			fadeInContainers[i].lock();
		}

		// full volume
		fullContainer = new AtomContainer();
		fullData = new TweenData(EndianOrder
				.flipNativeToBigEndian16((short) 2550), EndianOrder
				.flipNativeToBigEndian16((short) 2550));
		fullChildAtom = new Atom(
				StdQTConstants.kParentAtomIsContainer);
		fullAtom = fullContainer.insertChild(fullChildAtom,
				StdQTConstants.kTweenEntry, 1, 0);
		tempAtom5 = fullContainer
				.insertChild(
						fullAtom,
						StdQTConstants.kTweenType,
						1,
						0,
						EndianOrder
								.flipNativeToBigEndian32(StdQTConstants.kTweenTypeShort));
		tempAtom6 = fullContainer.insertChild(fullAtom,
				StdQTConstants.kTweenData, 1, 0, fullData);
		fullContainer.lock();

		// fade out
		fadeOutContainers = new AtomContainer[exponentialSteps];
		fadeOutData = new TweenData[fadeOutContainers.length];
		fadeOutAtoms = new Atom[fadeOutContainers.length];
		tempAtoms7 = new Atom[fadeOutContainers.length];
		tempAtoms8 = new Atom[fadeOutContainers.length];
		fadeOutChildAtoms = new Atom[fadeOutContainers.length];
		for (int i = 0; i < fadeOutContainers.length; i++) {
			fadeOutContainers[i] = new AtomContainer();
			short hi = EndianOrder
					.flipNativeToBigEndian16((short) ((Math.pow(3,
							(exponentialSteps - i) / (double) exponentialSteps) - 1) * 2550 * 0.5));
			short lo = EndianOrder
					.flipNativeToBigEndian16((short) ((Math.pow(3,
							(exponentialSteps - i - 1)
									/ (double) exponentialSteps) - 1) * 2550 * 0.5));
			fadeOutData[i] = new TweenData(hi, lo);
			fadeOutChildAtoms[i] = new Atom(
					StdQTConstants.kParentAtomIsContainer);
			fadeOutAtoms[i] = fadeOutContainers[i].insertChild(fadeOutChildAtoms[i],
					StdQTConstants.kTweenEntry, 1, 0);
			tempAtoms7[i] = fadeOutContainers[i]
					.insertChild(
							fadeOutAtoms[i],
							StdQTConstants.kTweenType,
							1,
							0,
							EndianOrder
									.flipNativeToBigEndian32(StdQTConstants.kTweenTypeShort));
			tempAtoms8[i] = fadeOutContainers[i].insertChild(fadeOutAtoms[i],
					StdQTConstants.kTweenData, 1, 0, fadeOutData[i]);
			fadeOutContainers[i].lock();
		}

		// tween durations
		int start = destStart;
		int fadein = (int) (dur * fadePercent);
		int fadeinmod = fadein - (fadein / exponentialSteps * exponentialSteps);
		int[] fadeins = new int[exponentialSteps];
		for (int i = 0; i < exponentialSteps; i++) {
			if (i < fadeinmod)
				fadeins[i] = fadein / exponentialSteps + 1;
			else
				fadeins[i] = fadein / exponentialSteps;
		}
		int fadeout = fadein - Mixer.SILENCE_BUFFER_IN_TIMEBASE;
		int fadeoutmod = fadeout
				- (fadeout / exponentialSteps * exponentialSteps);
		int[] fadeouts = new int[exponentialSteps];
		for (int i = 0; i < exponentialSteps; i++) {
			if (i < fadeoutmod)
				fadeouts[i] = fadeout / exponentialSteps + 1;
			else
				fadeouts[i] = fadeout / exponentialSteps;
		}
		int end = Mixer.SILENCE_BUFFER_IN_TIMEBASE;
		int full = dur - fadein - fadeout - end;

		// link the tween track to the audio track
		faderTrack = result.addTrack(0, 0, 0);
		handle = new QTHandle();
		ref = new DataRef(handle);
		ref.concatenate(silenceContainer);
		for (int i = 0; i < exponentialSteps; i++)
			ref = new DataRef(fadeInContainers[i]);
		ref.concatenate(fullContainer);
		for (int i = 0; i < exponentialSteps; i++)
			ref = new DataRef(fadeOutContainers[i]);
		ref.concatenate(silenceContainer);
		ref.lock();
		handle.lock();
		faderMedia = new TweenMedia(faderTrack, result.getTimeScale(), ref);
		sampleDescription = new SoundDescription(0);
		sampleDescription.lock();
		unlockHandles();
		faderMedia.beginEdits();
		faderMedia.addSample(silenceContainer, 0, silenceContainer.getSize(),
				start, sampleDescription, 1, 0);
		for (int i = 0; i < exponentialSteps; i++)
			faderMedia.addSample(fadeInContainers[i], 0, fadeInContainers[i]
					.getSize(), fadeins[i], sampleDescription, 1, 0);
		faderMedia.addSample(fullContainer, 0, fullContainer.getSize(), full,
				sampleDescription, 1, 0);
		for (int i = 0; i < exponentialSteps; i++)
			faderMedia.addSample(fadeOutContainers[i], 0, fadeOutContainers[i]
					.getSize(), fadeouts[i], sampleDescription, 1, 0);
		faderMedia.addSample(silenceContainer, 0, silenceContainer.getSize(),
				end, sampleDescription, 1, 0);
		faderMedia.endEdits();
		faderTrack.insertMedia(0, 0, start + dur, 1);
		int referenceIndex = destTrack.addReference(faderTrack,
				StdQTConstants.kTrackModifierReference);
		inputMap = new AtomContainer();
		inputMapChildAtom = new Atom(
				StdQTConstants.kParentAtomIsContainer);
		inputAtom = inputMap.insertChild(inputMapChildAtom,
				StdQTConstants.kTrackModifierInput, referenceIndex, 0);
		inputTemp1 = inputMap
				.insertChild(
						inputAtom,
						StdQTConstants.kTrackModifierType,
						1,
						0,
						EndianOrder
								.flipNativeToBigEndian32(StdQTConstants.kTrackModifierTypeVolume));
		inputTemp2 = inputMap.insertChild(inputAtom,
				StdQTConstants.kInputMapSubInputID, 1, 0, EndianOrder
						.flipNativeToBigEndian32(1));
		inputMap.lock();
		destTrack.getMedia().setInputMap(inputMap);
	}

	private void unlockHandles() {
		try {
			for (int i = 0; i < fadeInContainers.length; i++)
				fadeInContainers[i].unlock();
			for (int i = 0; i < fadeOutContainers.length; i++)
				fadeOutContainers[i].unlock();
			silenceContainer.unlock();
			fullContainer.unlock();
		} catch (Exception e) {
			// ignore
		}
	}

	public void cleanup() {
		try {
			ref.unlock();
			handle.unlock();
			sampleDescription.unlock();
			inputMap.unlock();
		} catch (Exception e) {
			// ignore
		}
	}
}
