package net.jasonfreeman.itsm.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Total hack of a class to deal with a special case in QTJ Windows installs where user installs
 * QTJ BEFORE installing a Java VM, so the QTJ files never end up in the JVM's class path. Basically,
 * we look for QTJava.zip and QTJava.dll in the standard places we'd expect to find them, and then
 * we manually copy them over to the proper directories in the JVM folder. Fun fun fun!
 * 
 * @author jason
 *
 */

public class ClassPathChecker {

	public static boolean fixQTJInstallation() {
		File zip = lookforFile("QTJava.zip");
		if (zip != null) {
			File dest = new File(System.getProperty("java.home") + "\\lib\\ext\\QTJava.zip");
			try {
				copyFile(zip, dest);
			} catch (IOException e) {
				return false;
			}
		} else {
			return false;
		}
		File dll = lookforFile("QTJava.dll");
		if (dll != null) {
			File dest = new File(System.getProperty("java.home") + "\\bin\\QTJava.dll");
			try {
				copyFile(dll, dest);
			} catch (IOException e) {
				return false;
			}
		} else {
			return false;
		}
		return true;
	}

	private static File lookforFile(String fileName) {
		String qtjEnv = System.getenv("QTJAVA");
		File qtjava = null;
		if (qtjEnv != null) {
			qtjava = new File(System.getenv("QTJAVA")).getParentFile();
			qtjava = findFile(qtjava, fileName);
		}
		if (qtjava == null) {
			qtjava = findFile(new File("C:\\Program Files\\Quicktime\\"), fileName);
			if (qtjava == null) {
				qtjava = findFile(new File("C:\\WINDOWS\\system\\"), fileName);
				if (qtjava == null) {
					qtjava = findFile(new File("C:\\WINDOWS\\system32\\"), fileName);
				}
			}
		}
		return qtjava;
	}
	
	private static File findFile(File path, String string) {
		if (path.isDirectory()) {
			File[] files = path.listFiles();
			if (files == null)
				return null;
			for (int i=0; i < files.length; i++) {
				File result = findFile(files[i], string);
				if (result != null)
					return result;
			}
			return null;
		} else {
			if (path.getName().endsWith(string))
				return path;
			else
				return null;
		}
	}

	private static void copyFile(File src, File dst) throws IOException {
		InputStream in = new FileInputStream(src);
        OutputStream out = new FileOutputStream(dst);
        byte[] buf = new byte[1024];
        int len;
        while ((len = in.read(buf)) > 0) {
            out.write(buf, 0, len);
        }
        in.close();
        out.close();
	}	
}
