/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.util;

import quicktime.QTSession;
import quicktime.util.QTBuild;

public class QTJavaCheck {

    /**
     * 
     * @return true if running compatible versions of Java and quicktime
     */
    public static boolean checkQTJVersion() {
    		boolean qtjGood = false;
        try {
            // first, catch the evil expired version of QTJ for Windows...
        	String OSname = System.getProperty("os.name");
        	if (OSname.startsWith("Windows") &&
        			QTBuild.info().equals("[QTJava:6.1.1a1]"))
        		return false;
        	// now, check everything else the normal way...
        	QTSession.open( );
            qtjGood = (QTSession.getMajorVersion() >= 6) &&
            			(QTBuild.getVersion() >= 6);
            QTSession.close( );
        } catch (Exception e) {
        	e.printStackTrace();
            return false;
        }
        return qtjGood;
    }
    
    public static boolean checkJavaVersion() {
        double javaVersion = convertVersionString(System.getProperty("java.version"));
        //System.out.println("javaVersion = " + javaVersion);
        return(javaVersion >= 1.3);
    }

    public static boolean checkOS() {
        String OSname = System.getProperty("os.name");
        double OSversion = convertVersionString(System.getProperty("os.version"));
        //System.out.println(OSname + " " + OSversion);
        return (OSname.equals("Mac OS X") || 
        		(OSname.startsWith("Windows") && (OSversion >= 5.0)));
    }
    
    private static double convertVersionString(String version) {
        try {
            int firstPeriod = version.indexOf('.');
            if (firstPeriod > 0) {
                int secondPeriod = version.indexOf('.', firstPeriod + 1);
                if (secondPeriod > 0) {
                		version = version
                            .substring(0, secondPeriod);
                }
            }
            return Double.valueOf(version).doubleValue();
        } catch (NumberFormatException enf) {
            return 0;
        }
    }
    
}
